﻿using System;
using System.Collections.Generic;
using System.Linq;
using VCommon.Ioc;
using VCommon.VApplication.Authorization;
using VCommonCoreExample.Caching;
using VCommonCoreExample.Configuration;

namespace VCommonCoreExample.Authorization
{
    public class PermissionProvider : ISingletonIocClass
    {
        private readonly AuthCache _authCache;

        private readonly PermissionTree _tree;
        private readonly HashSet<string> _allPermissionCodes;

        public PermissionProvider(AuthCache authCache)
        {
            _authCache = authCache;
            _tree = new PermissionTree();
            _allPermissionCodes = Init();
        }

        private HashSet<string> Init()
        {
            var nodePlatform = _tree.CreateRootNode(PermissionCodes.Platform.EntryPlatform, "平台管理", side: MultiTenancySides.Host);
            {
                //var nodeLic = nodePlatform.CreateChildNode(PermissionCodes.Platform.License.EntryLicense, "许可", side: MultiTenancySides.Host);
                //nodeLic.CreateChildNode(PermissionCodes.Platform.License.Query, "查询", side: MultiTenancySides.Host);
                //nodeLic.CreateChildNode(PermissionCodes.Platform.License.Create, "创建", side: MultiTenancySides.Host);
                //nodeLic.CreateChildNode(PermissionCodes.Platform.License.Update, "修改", side: MultiTenancySides.Host);
                //nodeLic.CreateChildNode(PermissionCodes.Platform.License.Delete, "删除", side: MultiTenancySides.Host);

                var nodeTenant = nodePlatform.CreateChildNode(PermissionCodes.Platform.Tenant.EntryTenant, "租户管理", side: MultiTenancySides.Host);
                nodeTenant.CreateChildNode(PermissionCodes.Platform.Tenant.Query, "查询", side: MultiTenancySides.Host);
                nodeTenant.CreateChildNode(PermissionCodes.Platform.Tenant.Create, "创建", side: MultiTenancySides.Host);
                nodeTenant.CreateChildNode(PermissionCodes.Platform.Tenant.Update, "修改", side: MultiTenancySides.Host);
                nodeTenant.CreateChildNode(PermissionCodes.Platform.Tenant.Delete, "删除", side: MultiTenancySides.Host);
            }

            var nodeSystem = _tree.CreateRootNode(PermissionCodes.System.EntrySystem, "系统管理", side: MultiTenancySides.Both);
            {
                var nodeRole = nodeSystem.CreateChildNode(PermissionCodes.System.Role.EntryRole, "角色管理", side: MultiTenancySides.Both);
                nodeRole.CreateChildNode(PermissionCodes.System.Role.Query, "查询", side: MultiTenancySides.Both);
                nodeRole.CreateChildNode(PermissionCodes.System.Role.Create, "创建", side: MultiTenancySides.Both);
                nodeRole.CreateChildNode(PermissionCodes.System.Role.Update, "修改", side: MultiTenancySides.Both);
                nodeRole.CreateChildNode(PermissionCodes.System.Role.Delete, "删除", side: MultiTenancySides.Both);

                var nodeOrg = nodeSystem.CreateChildNode(PermissionCodes.System.Org.EntryOrg, "组织管理", side: MultiTenancySides.Tenant);
                nodeOrg.CreateChildNode(PermissionCodes.System.Org.Query, "查询", side: MultiTenancySides.Tenant);
                nodeOrg.CreateChildNode(PermissionCodes.System.Org.Create, "创建", side: MultiTenancySides.Tenant);
                nodeOrg.CreateChildNode(PermissionCodes.System.Org.Update, "修改", side: MultiTenancySides.Tenant);
                nodeOrg.CreateChildNode(PermissionCodes.System.Org.Delete, "删除", side: MultiTenancySides.Tenant);

                var nodeUser = nodeSystem.CreateChildNode(PermissionCodes.System.User.EntryUser, "用户管理", side: MultiTenancySides.Both);
                nodeUser.CreateChildNode(PermissionCodes.System.User.Query, "查询", side: MultiTenancySides.Both);
                nodeUser.CreateChildNode(PermissionCodes.System.User.Create, "创建", side: MultiTenancySides.Both);
                nodeUser.CreateChildNode(PermissionCodes.System.User.Update, "修改", side: MultiTenancySides.Both);
                nodeUser.CreateChildNode(PermissionCodes.System.User.Delete, "删除", side: MultiTenancySides.Both);
            }

            return _tree.CompleteAdding();
        }

        public bool ValidPermission(string permission) => _allPermissionCodes.Contains(permission);

        public bool CheckPermission(Guid? tenantId, Guid userId, params string[] hasAnyPermission)
        {
            var ucm = _authCache.GetUser(tenantId, userId);
            if (ucm == null) return false;
            if (ucm.LoginName == ProjectConst.BuildInAdminUserName) return true;
            var permissions = ucm.Roles.SelectMany(p => _authCache.GetRole(tenantId, p).PermissionSet).ToHashSet();
            return hasAnyPermission.Any(permissions.Contains);
        }

        public IReadOnlyCollection<string> GetUserPermissionCodes(Guid? tenantId, Guid userId)
        {
            var ucm = _authCache.GetUser(tenantId, userId);
            if (ucm == null) return null;
            if (ucm.LoginName == ProjectConst.BuildInAdminUserName) return null;
            var permissions = ucm.Roles.SelectMany(p => _authCache.GetRole(tenantId, p).PermissionSet).ToHashSet();
            return permissions;
        }

        public IReadOnlyCollection<PermissionNodeOutput> GetPermissionTreeOutput(MultiTenancySides side, IReadOnlyCollection<string> licPermission = null)
        {
            return _tree.Filter(side, licPermission);
        }

      
    }
}
