﻿using Microsoft.Extensions.Configuration;
using System;
using System.Linq;
using VCommon.Caching;
using VCommon.Ioc;
using VCommon.VAutoMapper;
using VCommonCoreExample.Caching.Models;
using VCommonCoreExample.Configuration;
using VCommonCoreExample.Entity;
using VCommonCoreExample.EntityFrameworkCore;

namespace VCommonCoreExample.Caching
{
    public class AuthCache : ISingletonIocClass
    {
        //Tenant_T<TenantId>
        //  Enable -> bool
        //  User_<UserKey> -> {loginName,roles,password,enable}
        //  Role_<RoleId> -> {permissions}

        private const string EnableKey = "Enable";

        private readonly IIocManager _iocManager;
        private readonly FixedCacheManager _cache;

        public AuthCache(IConfiguration configuration, IIocManager iocManager)
        {
            _iocManager = iocManager;
            _cache = new FixedCacheManager(configuration[ProjectConst.CacheServerSettingKey], (int)CacheDbs.Auth);
        }

        private string TenantKey(Guid? tenantId) => "Tenant_T" + tenantId;

        private string UserKey(Guid userId) => "User_" + userId;

        private string RoleKey(Guid roleId) => "Role_" + roleId;

        public void Clear(Guid? tenantId) => _cache.KeyDelete(TenantKey(tenantId));

        public void ClearUser(Guid? tenantId, Guid userId) => _cache.HashFieldDelete(TenantKey(tenantId), UserKey(userId));

        public void ClearRole(Guid? tenantId, Guid roleId) => _cache.HashFieldDelete(TenantKey(tenantId), RoleKey(roleId));

        public bool? CheckTenantEnable(Guid tenantId)
        {
            return (bool?)_cache.HashFieldFetch(TenantKey(tenantId), EnableKey, () =>
            {
                using var db = _iocManager.Resolve<ExampleDbContext>();
                var value = db.Tenants.Where(p => p.IsAbolish == false && p.Id == tenantId)
                    .Select(p => (bool?)p.IsEnable).FirstOrDefault();
                return (CacheValue)value;
            });
        }

        public UserCacheModel GetUser(Guid? tenantId, Guid userId)
        {
            return _cache.HashFieldFetchJson(TenantKey(tenantId), UserKey(userId), () =>
            {
                using var db = _iocManager.Resolve<ExampleDbContext>();
                var cm = db.Users
                    .Where(p => p.IsAbolish == false && p.TenantId == tenantId && p.Id == userId)
                    .ProjectionFirstOrDefault<User, UserCacheModel>();
                return cm;
            });
        }

        public RoleCacheModel GetRole(Guid? tenantId, Guid roleId)
        {
            return _cache.HashFieldFetchJson(TenantKey(tenantId), RoleKey(roleId), () =>
            {
                using var db = _iocManager.Resolve<ExampleDbContext>();
                var cm = db.Roles
                    .Where(p => p.IsAbolish == false && p.TenantId == tenantId && p.Id == roleId)
                    .ProjectionFirstOrDefault<Role, RoleCacheModel>();
                return cm;
            });
        }
    }
}
