﻿using Microsoft.Extensions.Configuration;
using System;
using VCommon.Caching;
using VCommon.Ioc;
using VCommonCoreExample.Caching.Models;
using VCommonCoreExample.Configuration;

namespace VCommonCoreExample.Caching
{
    public class SessionCache : ISingletonIocClass
    {
        // Token_<Token> -> {tid,uid,password_hash,KickSign}
        // KickSign_<UserKey> -> match to UserLogin

        private readonly FixedCacheManager _cache;

        public SessionCache(IConfiguration configuration) => _cache = new FixedCacheManager(configuration[ProjectConst.CacheServerSettingKey], (int)CacheDbs.Session);

        private string TokenKey(string token) => "Token_" + token;

        private string KickSignKey(Guid? tenantId, Guid userId) => "KickSign_T" + tenantId + "_U" + userId;

        public void Clear(string token) => _cache.KeyDelete(TokenKey(token));

        public void Set(string token, SessionCacheModel value)
        {
            _cache.KeySetJson(TokenKey(token), value);
            _cache.KeySet(KickSignKey(value.TenantId, value.UserId), (CacheValue)value.KickSign);
        }

        public SessionCacheModel Get(string token) => _cache.KeyGetJson<SessionCacheModel>(TokenKey(token));

        public string GetKickSign(Guid? tenantId, Guid userId) => (string)_cache.KeyGet(KickSignKey(tenantId, userId));

        public void ExtendExpire(string token, Guid? tenantId, Guid userId, int days)
        {
            _cache.KeyExpire(TokenKey(token), new CacheExpire { Specified = TimeSpan.FromDays(days) });
            _cache.KeyExpire(KickSignKey(tenantId, userId), new CacheExpire { Specified = TimeSpan.FromDays(days) });
        }
    }
}
